class Logger:
    """
    The base class for loggers for use with Netcat objects. Each of these
    methods will be called to indicate a given event.
    """
    def connected(self, peer):
        """
        Called with a tuple of the peer to indicate "connection established",
        either as a client or a server
        """

    def sending(self, data):
        """
        Called to indicate that some data has been sent over the wire
        """

    def buffering(self, data):
        """
        Called to indicate that some data has been received and inserted into
        the buffer
        """

    def unbuffering(self, data):
        """
        Called to indicate that some data is being extracted from the buffer
        and returned to the user
        """

    def interrupted(self):
        """
        Called to indicate that a socket operation was interrupted via ctrl-c
        """

    def eofed(self):
        """
        Called to indicate that reading from the socket resulted in an EOF
        condition
        """

    def requesting_send(self, data):
        """
        Called to indicate that the user has asked to send all of some data
        """

    def requesting_recv(self, n, timeout):
        """
        Called to indicate that the user has asked for a receive of at most n
        bytes
        """

    def requesting_recv_until(self, s, max_size, timeout):
        """
        Called to indicate that the user has asked to receive until a given
        string appears
        """

    def requesting_recv_all(self, timeout):
        """
        Called to indicate that the user has asked to receive all data until
        close
        """

    def requesting_recv_exactly(self, n, timeout):
        """
        Called to indicate that the user has asked to receive exactly n bytes
        """

    def interact_starting(self):
        """
        Called to indicate that an interactive session is beginning
        """

    def interact_ending(self):
        """
        Called to indicate that an interactive session is ending
        """

class ManyLogger(Logger):
    """
    A logger which dispatches all events to all its children, which are other
    loggers. You shouldn't have to deal with this much; it's used automatically
    by the Netcat.

    :param children:        A list of loggers to which to dispatch events
    """
    def __init__(self, children):
        self.children = children

    def connected(self, peer):
        for child in self.children:
            child.connected(peer)

    def sending(self, data):
        for child in self.children:
            child.sending(data)

    def buffering(self, data):
        for child in self.children:
            child.buffering(data)

    def unbuffering(self, data):
        for child in self.children:
            child.unbuffering(data)

    def interrupted(self):
        for child in self.children:
            child.interrupted()

    def eofed(self):
        for child in self.children:
            child.eofed()

    def requesting_send(self, data):
        for child in self.children:
            child.requesting_send(data)

    def requesting_recv(self, n, timeout):
        for child in self.children:
            child.requesting_recv(n, timeout)

    def requesting_recv_until(self, s, max_size, timeout):
        for child in self.children:
            child.requesting_recv_until(s, max_size, timeout)

    def requesting_recv_all(self, timeout):
        for child in self.children:
            child.requesting_recv_all(timeout)

    def requesting_recv_exactly(self, n, timeout):
        for child in self.children:
            child.requesting_recv_exactly(n, timeout)

    def interact_starting(self):
        for child in self.children:
            child.interact_starting()

    def interact_ending(self):
        for child in self.children:
            child.interact_ending()

class TeeLogger(Logger):
    """
    A logger which feeds a copy of the input and output streams to a given stream

    :param log_send:    A simplesock object to log all sends to, or None
    :param log_recv:    A simplesock object to log all recvs to, or None
    :param log_yield:   Whether recv logging should happen when data is
                        buffered or returned to the user
    """
    def __init__(self, log_send=None, log_recv=None, log_yield=False):
        self.log_send = log_send
        self.log_recv = log_recv
        self.log_yield = log_yield

    def sending(self, data):
        if self.log_send is not None:
            self.log_send.send(data)

    def buffering(self, data):
        if not self.log_yield and self.log_recv is not None:
            self.log_recv.send(data)

    def unbuffering(self, data):
        if self.log_yield and self.log_recv is not None:
            self.log_recv.send(data)

class StandardLogger(Logger):
    """
    A logger which produces a human-readable log of what's happening

    :param log:             A simplesock object to which the logs should
                            be sent
    :param log_yield:       Whether to log receives when they are recieved
                            and written to the buffer or when they are
                            unbuffered and returned to the user
    :param show_headers:    Whether to show metadata notices about user
                            actions and exceptional conditions
    :param hex_dump:        Whether to encode logged data as a canonical
                            hexdump
    :param split_newlines:  When in non-hex mode, whether logging should
                            treat newlines as record separators
    """
    def __init__(self, log, log_yield=False, show_headers=True,
            hex_dump=False, split_newlines=True,
            send_prefix='>> ', recv_prefix='<< ', no_eol_indicator='\x1b[3m%\x1b[0m'):
        self.log = log
        self.log_yield = log_yield
        self.show_headers = show_headers
        self.hex_dump = hex_dump
        self.split_newlines = split_newlines
        self.send_prefix = send_prefix
        self.recv_prefix = recv_prefix
        self.no_eol_indicator = no_eol_indicator

        self.suppressed = False
        self.counter_send = 0
        self.counter_recv = 0

    def _log(self, s):
        self.log.send(s.encode())

    def _header(self, s):
        if self.show_headers:
            self._log('======= %s =======\n' % s)

    def sending(self, data):
        if self.suppressed:
            return
        self._log_data(data, self.counter_send, self.send_prefix)
        self.counter_send += len(data)

    def _recving(self, data):
        if self.suppressed:
            return
        self._log_data(data, self.counter_recv, self.recv_prefix)
        self.counter_recv += len(data)

    def _log_data(self, data, counter, prefix):
        if not data:
            return

        if self.hex_dump:
            line_progress = counter % 16
            first_line_size = 16 - line_progress
            first_line = data[:first_line_size]
            self._log(prefix + self._hex_line(counter, first_line))

            for i in range(first_line_size, len(data), 16):
                line = data[i:i+16]
                self._log(prefix + self._hex_line(counter + i, line))

        else:
            noeol = False
            if self.split_newlines:
                records = data.split(b'\n')
                if len(records) > 1 and records[-1] == b'':
                    records.pop()
                else:
                    noeol = True
            else:
                records = [data]

            for i, record in enumerate(records):
                sep = (self.no_eol_indicator if noeol and i == len(records) - 1 else '') + '\n'
                self._log(prefix + self._escape(record) + sep)

    @staticmethod
    def _hex_line(counter, line):
        advance = counter % 16
        tail = 16 - advance - len(line)
        lhex = line.hex().upper()
        lhex = '  '*advance + lhex + '  '*tail
        lascii = ' '*advance + ''.join(chr(c) if 0x20 <= c <= 0x7e else '.' for c in line) + ' '*tail

        fargs = (counter,) + tuple(lhex[i:i+2] for i in range(0, 0x20, 2)) + (lascii,)
        return '%06X  %s %s %s %s %s %s %s %s  %s %s %s %s %s %s %s %s  |%s|\n' % fargs

    @staticmethod
    def _escape(bs):
        return ''.join(StandardLogger._escchr(c) for c in bs)

    @staticmethod
    def _escchr(c):
        if c == ord('\\'):
            return '\\\\'
        if c == ord('\n'):
            return '\\n'
        if c == ord('\t'):
            return '\\t'
        if c < 0x20 or c > 0x7e:
            return '\\x%02x' % c
        return chr(c)

    def buffering(self, data):
        if not self.log_yield:
            self._recving(data)

    def unbuffering(self, data):
        if self.log_yield:
            self._recving(data)

    def connected(self, peer):
        self._header("Connected to %s" % str(peer))

    def interrupted(self):
        self._header("Connection interrupted")

    def eofed(self):
        self._header("Received EOF")

    def requesting_send(self, data):
        if self.suppressed:
            return
        self._header("Sending %d byte%s" % (len(data), '' if len(data) == 1 else 's'))

    @staticmethod
    def _timeout_text(timeout):
        if timeout is None:
            return ''
        if timeout == 0:
            return ' (nonblocking)'
        return ' (until %s second%s)' % (timeout, '' if timeout == 1 else 's')

    def requesting_recv(self, n, timeout):
        if self.suppressed:
            return
        self._header("Receiving at most %d byte%s%s" % (n, '' if n == 1 else 's', self._timeout_text(timeout)))

    def requesting_recv_until(self, s, max_size, timeout):
        if self.suppressed:
            return
        if max_size is not None:
            max_size_text = ', max of %d byte%s' % (max_size, '' if max_size == 1 else 's')
        else:
            max_size_text = ''

        self._header("Receiving until %s%s%s" % (repr(s).strip('b'), max_size_text, self._timeout_text(timeout)))

    def requesting_recv_all(self, timeout):
        if self.suppressed:
            return
        self._header("Receiving until close%s" % self._timeout_text(timeout))

    def requesting_recv_exactly(self, n, timeout):
        self._header("Receiving exactly %d byte%s%s" % (n, '' if n == 1 else 's', self._timeout_text(timeout)))

    def interact_starting(self):
        self._header("Beginning interactive session")
        self.suppressed = True

    def interact_ending(self):
        self.suppressed = False



import socket
import io
import logging

from .select import select
from .errors import NetcatError

class Simple:
    """
    The base class for implementing a simple, unified interface for socket-like
    objects. Instances of this type should act like a simple unbuffered
    blocking socket.

    :ivar can_send: Whether this stream supports send operations
    :ivar can_recv: Whether this stream supports recv operations
    """
    def __init__(self):
        self.can_send = False
        self.can_recv = False

    def recv(self, size):
        """
        Receive at most `size` bytes, blocking until some data is available.
        Returns an empty bytestring as an EOF condition.
        """
        raise NotImplementedError

    def send(self, data):
        """
        Send as much of the given data as possible, returning the number of
        bytes sent.
        """
        raise NotImplementedError

    def close(self):
        """
        Close the stream.
        """
        raise NotImplementedError

    @property
    def closed(self):
        """
        Whether the stream has been closed.
        """
        raise NotImplementedError

    def fileno(self):
        """
        The file descriptor associated with the stream. Should raise
        `NetcatError` if there is not a single file descriptor to return.
        """
        raise NotImplementedError

    def shutdown(self, how):
        """
        Indicate somehow that there will be no more reading/writing/both to
        this stream. Takes the ``socket.SHUT_*`` constants.
        """
        raise NotImplementedError

    def _prep_select(self):
        """
        Return three tuples of all the raw python file objects that should be
        selected over in order to determine if this stream is
        readable/writable/in an exceptional condition.
        """
        raise NotImplementedError

def wrap(sock):
    """
    A helper function to automatically pick the correct wrapper class for a
    sock-like object.
    """
    if isinstance(sock, Simple):
        return sock
    if isinstance(sock, socket.socket):
        return SimpleSocket(sock)
    if isinstance(sock, io.IOBase):
        return SimpleFile(sock)
    raise ValueError("idk how to work with a %r. check your work or report a bug?" % type(sock))

class SimpleSocket(Simple):
    """
    A wrapper for sockets.

    :param sock:    A python ``socket.socket`` object.
    """
    def __init__(self, sock):
        super().__init__()
        self.sock = sock  # a socket.socket object
        self.can_recv = True
        self.can_send = True

        # disable timeout and enable blocking. this is a simple socket after all.
        sock.settimeout(None)
        sock.setblocking(True)

    def recv(self, size):
        return self.sock.recv(size)

    def send(self, data):
        return self.sock.send(data)

    def close(self):
        return self.sock.close()

    @property
    def closed(self):
        return self.sock._closed

    def fileno(self):
        return self.sock.fileno()

    def shutdown(self, how):
        try:
            return self.sock.shutdown(how)
        except OSError:
            # e.g. udp sockets may do this
            pass

    def _prep_select(self):
        return ((self.sock,) if self.can_recv else ()), ((self.sock,) if self.can_send else ()), (self.sock,)

class SimpleFile(Simple):
    """
    A wrapper for files from the python ``io`` module, including ``sys.stdin``,
    subprocess pipes, etc. If the file has an encoding, it will be discarded.

    :param fp:      An ``io.IOBase`` object
    """
    def __init__(self, fp):
        super().__init__()
        self.please_decode = None

        try: # check if we have a TextIOWrapper
            buf = fp.buffer
            self.please_decode = fp.encoding
            fp = buf
        except AttributeError:
            pass

        try: # check if we have a BufferedReader
            fp = fp.raw
        except AttributeError:
            pass

        self.file = fp  # an io.IOBase object
        self.can_recv = fp.mode.startswith('r')
        self.can_send = fp.mode.startswith('w') or fp.mode.startswith('a') or '+' in fp.mode

    def recv(self, size):
        if not self.can_recv:
            raise Exception("Unsupported operation")
        return self.file.read(size)

    def send(self, data):
        if not self.can_send:
            raise Exception("Unsupported operation")
        return self.file.write(data)

    def close(self):
        return self.file.close()

    @property
    def closed(self):
        return self.file.closed

    def fileno(self):
        return self.file.fileno()

    def shutdown(self, how):
        if how == socket.SHUT_RDWR:
            self.close()
        elif how == socket.SHUT_RD:
            if not self.can_send:
                self.close()
        elif how == socket.SHUT_WR:
            if not self.can_recv:
                self.close()
        else:
            raise ValueError("invalid shutdown `how`", how)

    def _prep_select(self):
        return ((self.file,) if self.can_recv else ()), ((self.file,) if self.can_send else ()), (self.file,)


class SimpleDuplex(Simple):
    """
    A wrapper which splits recv and send operations across two different
    streams.

    :param Simple child_recv:  The stream to use for recving
    :param Simple child_send: The stream to use for sending

    If either of these parameters are None, that operation will be disabled
    and generate exceptions.
    """

    def __init__(self, child_recv=None, child_send=None):
        super().__init__()
        self.child_recv = child_recv
        self.child_send = child_send

        self.can_recv = self.child_recv is not None
        self.can_send = self.child_send is not None

        if self.can_recv and not self.child_recv.can_recv:
            raise NetcatError("Reading sock cannot be used for recving")
        if self.can_send and not self.child_send.can_send:
            raise NetcatError("Sending sock cannot be used for sending")

    def recv(self, size):
        if not self.can_recv:
            raise NetcatError("Unsupported operation")
        return self.child_recv.recv(size)

    def send(self, data):
        if not self.can_send:
            raise NetcatError("Unsupported operation")
        return self.child_send.send(data)

    def close(self):
        if self.can_recv:
            self.child_recv.close()
        if self.can_send:
            self.child_send.close()

    @property
    def closed(self):
        if self.can_recv:
            return self.child_recv.closed
        if self.can_send:
            return self.child_send.closed
        return True

    def fileno(self):
        if self.can_recv:
            if self.can_send:
                raise NetcatError("Socket has multiple filenos")
            return self.child_recv.fileno()
        if self.can_send:
            return self.child_send.fileno()
        raise NetcatError("Socket has no filenos")

    def shutdown(self, how):
        # should these filter the how value so recv never sees a send shutdown etc?
        if self.can_recv:
            self.child_recv.shutdown(how)
        if self.can_send:
            self.child_send.shutdown(how)

    def _prep_select(self):
        rfd = ((), (), ()) if self.child_recv is None else self.child_recv._prep_select()
        wfd = ((), (), ()) if self.child_send is None else self.child_send._prep_select()

        return (rfd[0] if self.can_recv else ()), (wfd[1] if self.can_send else ()), rfd[2] + wfd[2]

class SimpleMerge(Simple):
    """
    A stream that merges the output of many readable streams into one.

    :param children:    A list of streams from which to read
    :type children:     list of Simple
    """
    def __init__(self, children):
        super().__init__()
        self.can_send = False
        self.can_recv = True
        self.children = children

        if any(not child.can_recv for child in children):
            raise Exception("Not all children are applicable for receiving")

    def recv(self, size):
        goodsocks, _, _ = select(self.children)
        if not goodsocks:
            raise Exception("What happened???")

        goodsock = goodsocks[0]
        return goodsock.recv(size)

    def send(self, data):
        raise Exception("Cannot send to a merged socket")

    def close(self):
        for child in self.children:
            child.close()

    @property
    def closed(self):
        # TODO: consistency check?
        return any(child.closed for child in self.children)

    def fileno(self):
        raise Exception("Socket has multiple filenos")

    def shutdown(self, how):
        for child in self.children:
            child.shutdown(how)

    def _prep_select(self):
        stuff = sum((child._prep_select()[0] for child in self.children), ())
        return stuff, (), stuff

class SimpleNetcat(Simple):
    """
    A wrapper for a Netcat object! Why? Just in case you want to do
    Netcat-level instrumentation [logging] at a finer granularity than the
    top-level.

    :param sock:    A Netcat object.
    """
    def __init__(self, nc):
        super().__init__()
        self.nc = nc
        self.can_recv = nc.sock.can_recv
        self.can_send = nc.sock.can_send

        nc.settimeout(None)

    def recv(self, size):
        return self.nc.recv(size)

    def send(self, data):
        return self.nc.send(data)

    def close(self):
        return self.nc.close()

    @property
    def closed(self):
        return self.nc.closed

    def fileno(self):
        return self.nc.fileno()

    def shutdown(self, how):
        return self.nc.shutdown(how)

    def _prep_select(self):
        return self.nc._prep_select()

class SimpleLogger(Simple):
    """
    A socket-like interface for dumping data to a python logging endpoint.

    :param logger:      The dotted name for the endpoint for the logs to go to
    :param level:       The string or numeric severity level for the logging
                        messages
    :param encoding:    simplesock objects are fed bytestrings. Loggers consume
                        unicode strings. How should we translate?
    """
    def __init__(self, logger='nclib.logs', level='INFO', encoding=None):
        super().__init__()
        self.logger = logging.getLogger(logger)
        self.level = logging._checkLevel(level)
        self.encoding = encoding
        self._closed = False

        self.can_recv = False
        self.can_send = True

    def recv(self, size):
        raise NetcatError("Can't recv from a logger object")

    def send(self, data):
        if self._closed:
            raise NetcatError("I'm closed dumbass!")

        if self.encoding is None:
            data = data.decode()
        else:
            data = data.decode(self.encoding)
        self.logger.log(self.level, data)

    def close(self):
        self._closed = True

    @property
    def closed(self):
        return self._closed

    def fileno(self):
        raise NetcatError("Can't make a fileno for you")

    def shutdown(self, how):
        return None

    def _prep_select(self):
        raise NetcatError("Can't be selected")



import subprocess
import os
import random

from .netcat import Netcat

class Process(Netcat):
    """
    A mechanism for launching a local process and interacting with it
    programatically.  This class is a subclass of the basic `Netcat` object so
    you may use any method from that class to interact with the process you've
    launched!

    :param program:     The program to launch. Can be either a list of strings,
                        in which case those strings will become the program
                        argv, or a single string, in which case the shell will
                        be used to launch the program.
    :param stderr:      How the program's stderr stream should behave. True
                        (default) will redirect stderr to the output socket,
                        unifying it with stdout. False will redirect it to
                        /dev/null. None will not touch it, causing it to appear
                        on your terminal.
    :param cwd:         The working directory to execute the program in
    :param env:         The environment to execute the program in, as a
                        dictionary

    Any additional keyword arguments will be passed to the constructor of
    Netcat.

    WARNING: If you provide a string and not a list as the description for the
    program to launch, then the pid we know about will be associated with the
    shell that launches the program, not the program itself.

    *Example:* Launch the `cat` process and send it a greeting. Print out its
    response. Close the socket and the process exits with status 0.

    >>> from nclib import Process
    >>> cat = Process('cat')
    >>> cat.send('Hello world!')
    >>> print(cat.recv())
    b'Hello world!'
    >>> cat.close()
    >>> print(cat.poll())
    0
    """
    def __init__(self, program,
            stderr=True,
            cwd=None,
            env=None,
            **kwargs):
        self._subprocess = self.launch(program, stderr=stderr, cwd=cwd, env=env)
        self.pid = self._subprocess.pid

        super().__init__(
                sock=self._subprocess.stdout,
                sock_send=self._subprocess.stdin,
                server='local program %s' % program, **kwargs)

    def poll(self):
        """
        Return the exit code of the proces, or None if it has not exited.
        """
        return self._subprocess.poll()

    def wait(self):
        """
        Wait for the process to exit and return its exit code.
        """
        return self._subprocess.wait()

    def send_signal(self, sig):
        """
        Send the signal `sig` to the process.
        """
        return self._subprocess.send_signal(sig)

    def kill(self):
        """
        Terminate the process.
        """
        return self._subprocess.kill()

    @staticmethod
    def launch(program, sock=None, stderr=True, cwd=None, env=None):
        """
        A static method for launching a process.
        Same rules from the Process constructor apply.

        :param sock:    The socket to use for stdin/stdout. If None, will be a new set of pipes.
        """

        if stderr is True:
            stderr = subprocess.STDOUT
        elif stderr is False:
            stderr = subprocess.DEVNULL

        if sock is None:
            sock = subprocess.PIPE

        return subprocess.Popen(program,
                shell=type(program) not in (list, tuple),
                stdin=sock, stdout=sock, stderr=stderr,
                cwd=cwd, env=env,
                close_fds=True)


class GDBProcess(Process):
    """
    Like nclib.Process, but also launches gdb (in a new $TERMINAL, by default gnome-terminal)
    to debug the process.
    """
    def __init__(self, program, gdbscript=None, **kwargs):
        """
        :param program:     The program to launch. Can be either a list of strings, in which case
                            those strings will become the program argv, or a single string, in which
                            case the shell will be used to launch the program.
        :param stderr:      How the program's stderr stream should behave. True (default) will
                            redirect stderr to the output socket, unifying it with stdout. False will
                            redirect it to /dev/null. None will not touch it, causing it to appear
                            on your terminal.
        :param cwd:         The working directory to execute the program in
        :param env:         The environment to execute the program in, as a dictionary
        :param protocol:    The socket protocol to use. 'tcp' by default, can also be 'udp'
        :param gdbscript:   The filename of a script for gdb to execute automatically on startup

        Any additional keyword arguments will be passed to the constructor of Netcat.
        """
        super(GDBProcess, self).__init__(program, **kwargs)

        progbase = (program.split() if type(program) not in (list, tuple) else program)[0]
        # TODO: this should be refactored so it's the other way around
        # we should be fromatting stuff into bytestrings since that's what goes into the syscalls
        if bytes is not str and type(progbase) is bytes:
            progbase = progbase.decode()
        gdbcmd = 'gdb %s -ex "set sysroot" -ex "target remote tcp::%d"' % (progbase, self._subprocess._gdbport) # pylint: disable=no-member
        if gdbscript is not None:
            gdbcmd += " -x '%s'" % (gdbscript.replace("'", "'\"'\"'"))

        nul = open(os.devnull, 'r+b')
        t = os.environ.get("TERMINAL", "gnome-terminal").split()
        self.term = subprocess.Popen(t + ['-e', gdbcmd],
                close_fds=True,
                stdin=nul, stdout=nul, stderr=nul)

        self.recv_until(b'pid = ')
        self.pid = int(self.recvline())
        self.recvline()
        self.recvline()

    @classmethod
    def launch(cls, program, *args, **kwargs):
        gdbport = random.randint(32768, 60999) # default /proc/sys/net/ipv4/ip_local_port_range on my machine
        gdbcmd = ['gdbserver', 'localhost:%d' % gdbport]
        if type(program) not in (list, tuple):
            if bytes is not str and type(program) is bytes:
                program = program.decode()
            program = '%s %s' % (' '.join(gdbcmd), program)
        else:
            program = gdbcmd + program

        p = super(GDBProcess, cls).launch(program, *args, **kwargs)
        p._gdbport = gdbport
        return p



long_description = '''
nclib is a python socket library that wants to be your friend.

nclib provides:

- Easy-to-use interfaces for connecting to and listening on TCP and UDP sockets
- The ability to handle any python stream-like object with a single interface
- A better socket class, the Netcat object

  - Convenient receive methods for common socket usage patterns
  - Highly customizable logging
  - Interactive mode, connecting the socket to your stdin/stdout
  - Intelligent detection of socket closes and connection drops
  - Long-running functions cleanly abortable with ctrl-c
  - Lots of aliases in case you forget the right method name

- Mechanisms to launch processes with their in/out streams connected to sockets

  - Launch a process with gdb attached

- TCP and UDP server classes for writing simple python daemons
- A script to easily daemonize command-line programs

Documentation is available at https://nclib.readthedocs.io/ and source code is
available at https://github.com/rhelmot/nclib

If you are familiar with pwntools, nclib provides much of the functionaly that
pwntools' socket wrappers do, but with the bonus feature of not being pwntools.
'''

if bytes is str:
  raise Exception("nclib is python 3 only now :(")

from setuptools import setup
setup(name='nclib',
      version='1.0.0.rc3',
      python_requires='>=3.5',
      packages=['nclib'],
      scripts=['serve-stdio'],
      description='Netcat as a library: convienent socket interfaces',
      long_description=long_description,
      url='https://github.com/rhelmot/nclib',
      author='rhelmot',
      author_email='audrey@rhelmot.io',
      license='MIT',
      keywords='netcat nc socket sock file pipe tcp udp recv until logging interact handle listen connect server serve stdio process gdb'
      )
